@@ -1,4 +1,6 @@
-{-# LANGUAGE Rank2Types #-}
+{-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE Rank2Types       #-}
+{-# LANGUAGE TypeFamilies     #-}
 
 -----------------------------------------------------------------------------
 -- |
@@ -13,44 +15,44 @@
 -----------------------------------------------------------------------------
 
 module Diagrams.TwoD.Adjust
-    (
-      setDefault2DAttributes
-    , adjustDiaSize2D
+    ( setDefault2DAttributes
+    , adjustSize2D
     , adjustDia2D
     ) where
 
-import           Diagrams.Attributes      (lineCap, lineJoin,
-                                           lineMiterLimitA)
+import           Diagrams.Attributes
 import           Diagrams.Core
-import           Diagrams.TwoD.Attributes (lineWidthA, lineTextureA)
-import           Diagrams.TwoD.Size       (SizeSpec2D (..), center2D,
-                                           requiredScale, size2D)
-import           Diagrams.TwoD.Text       (fontSizeA)
-import           Diagrams.TwoD.Types      (R2, T2, p2)
+import           Diagrams.TwoD.Attributes (lineTextureA)
+import           Diagrams.TwoD.Types
 import           Diagrams.Util            (( # ))
+import           Diagrams.Size
+import           Diagrams.BoundingBox
 
-import           Control.Lens             (Lens', (&), (.~), (^.))
-import           Data.AffineSpace         ((.-.))
+import           Control.Lens             (Lens', (^.),  set)
 import           Data.Default.Class
 import           Data.Semigroup
 
+
 -- | Set default attributes of a 2D diagram (in case they have not
 --   been set):
 --
---       * Line width 0.01
+--       * 'LineWidth':  0.01
 --
---       * Line color black
+--       * 'LineTexture': solid black
 --
---       * Font size 1
+--       * 'LineCap': LineCapButt
 --
---       * Line cap LineCapButt
+--       * 'LineJoin': miter
 --
---       * line join miter
---
---       * Miter limit 10
-setDefault2DAttributes :: Semigroup m => QDiagram b R2 m -> QDiagram b R2 m
-setDefault2DAttributes d = d # lineWidthA def # lineTextureA def # fontSizeA def
-                             # lineCap def # lineJoin def # lineMiterLimitA def
+--       * 'MiterLimit': 10
+setDefault2DAttributes :: (TypeableFloat n, Semigroup m)
+  => QDiagram b V2 n m -> QDiagram b V2 n m
+setDefault2DAttributes d
+  = d # lineWidthM def
+      # lineTextureA def
+      # lineCap def
+      # lineJoin def
+      # lineMiterLimitA def
 
 
 -- | Adjust the size and position of a 2D diagram to fit within the
@@ -60,25 +62,14 @@ setDefault2DAttributes d = d # lineWidthA def # lineTextureA def # fontSizeA def
 --   inverse of which can be used, say, to translate output/device
 --   coordinates back into local diagram coordinates), and the
 --   modified diagram itself.
-adjustDiaSize2D :: Monoid' m
-                => Lens' (Options b R2) SizeSpec2D
-                -> b -> Options b R2 -> QDiagram b R2 m
-                -> (Options b R2, T2, QDiagram b R2 m)
-adjustDiaSize2D szL _ opts d =
-  ( case spec of
-     Dims _ _ -> opts
-     _        -> opts & szL .~ (uncurry Dims . scale s $ size)
-  , adjustT
-  , d # transform adjustT
-  )
-  where spec = opts ^. szL
-        size = size2D d
-        s    = requiredScale spec size
-        finalSz = case spec of
-                    Dims w h -> (w,h)
-                    _        -> scale s size
-        tr = (0.5 *. p2 finalSz) .-. (s *. center2D d)
-        adjustT = translation tr <> scaling s
+adjustSize2D :: (TypeableFloat n, Monoid' m)
+                => Lens' (Options b V2 n) (SizeSpec V2 n)
+                -> b -> Options b V2 n -> QDiagram b V2 n m
+                -> (Options b V2 n, Transformation V2 n, QDiagram b V2 n m)
+adjustSize2D szL _ opts d = (set szL spec opts, t, d # transform t)
+  where
+    spec    = dims sz
+    (sz, t) = sizeAdjustment (opts ^. szL) (boundingBox d)
 
 -- | @adjustDia2D@ provides a useful default implementation of
 --   the 'adjustDia' method from the 'Backend' type class.
@@ -97,10 +88,10 @@ adjustDiaSize2D szL _ opts d =
 --   to the diagram (the inverse of which can be used, say, to
 --   translate output/device coordinates back into local diagram
 --   coordinates), and the modified diagram itself.
-adjustDia2D :: Monoid' m
-            => Lens' (Options b R2) SizeSpec2D
-            -> b -> Options b R2 -> QDiagram b R2 m
-            -> (Options b R2, T2, QDiagram b R2 m)
+adjustDia2D :: (TypeableFloat n, Monoid' m)
+            => Lens' (Options b V2 n) (SizeSpec V2 n)
+            -> b -> Options b V2 n -> QDiagram b V2 n m
+            -> (Options b V2 n, Transformation V2 n, QDiagram b V2 n m)
 adjustDia2D szL b opts d
-  = adjustDiaSize2D szL b opts (d # setDefault2DAttributes)
+  = adjustSize2D szL b opts (d # setDefault2DAttributes)
 
